namespace Unibas.MorraCinese.Test {

using NUnit.Framework;
using Unibas.MorraCinese;

[TestFixture]
public class TestPartita {
    
    private Partita partita;
    private int pareggi;
    private int vittorieComputer;
    private int vittorieGiocatore;
    
    public TestPartita () {
        System.Diagnostics.DefaultTraceListener listener
               = (System.Diagnostics.DefaultTraceListener)System.Diagnostics.Debug.Listeners[0];
        listener.AssertUiEnabled = false;
        System.Diagnostics.Debug.Listeners.Add(new Unibas.Utilita.MioTraceListener());
    }

    [SetUp]
    public void SetUp() {
        partita = new Partita();
        pareggi = 0;
        vittorieComputer = 0;
        vittorieGiocatore = 0;
    }
    
    [Test]
    public void TestPartita1() {
        VerificaPunteggi(0, 0, 0);
        VerificaManoCarta();
        VerificaManoForbici();
        VerificaManoSasso();
        while (partita.Stato == StatoPartita.PARTITAINCORSO) {
            VerificaManoCarta();
        }
        if (vittorieComputer > vittorieGiocatore) {
            Assert.IsTrue(partita.Stato == StatoPartita.PARTITAVINTADALCOMPUTER, "vince il computer");
            Assert.IsTrue(partita.ManiVinteDalComputer == Partita.LIMITEMANI, "punti computer");
        } else {
            Assert.IsTrue(partita.Stato == StatoPartita.PARTITAVINTADALGIOCATORE, "vince il computer");
            Assert.IsTrue(partita.ManiVinteDalGiocatore == Partita.LIMITEMANI, "punti giocatore");            
        }
    }
    
    [Test]
    public void TestPartita2() {
        VerificaPunteggi(0, 0, 0);
        while (partita.Stato == StatoPartita.PARTITAINCORSO) {
            VerificaManoCarta();
        }
        try {
            VerificaManoCarta();
            Assert.Fail();
        } catch (Unibas.Utilita.AssertionException) { }
        
    }

    private void VerificaManoCarta() {
        Mano mano = new Mano();
        mano.Gioca(Oggetto.CARTA);
        partita.GestisciMano(mano);
        if (mano.GiocataComputer == Oggetto.CARTA) {
            VerificaPunteggi(++pareggi, vittorieComputer, vittorieGiocatore);
        } else if (mano.GiocataComputer == Oggetto.FORBICI) {
            VerificaPunteggi(pareggi, ++vittorieComputer, vittorieGiocatore);
        } else if (mano.GiocataComputer == Oggetto.SASSO) {
            VerificaPunteggi(pareggi, vittorieComputer, ++vittorieGiocatore);
        }        
    }
    
    private void VerificaManoForbici() {
        Mano mano = new Mano();
        mano.Gioca(Oggetto.FORBICI);
        partita.GestisciMano(mano);
        if (mano.GiocataComputer == Oggetto.CARTA) {
            VerificaPunteggi(pareggi, vittorieComputer, ++vittorieGiocatore);
        } else if (mano.GiocataComputer == Oggetto.FORBICI) {
            VerificaPunteggi(++pareggi, vittorieComputer, vittorieGiocatore);
        } else if (mano.GiocataComputer == Oggetto.SASSO) {
            VerificaPunteggi(pareggi, ++vittorieComputer, vittorieGiocatore);
        }        
    }

    private void VerificaManoSasso() {
        Mano mano = new Mano();
        mano.Gioca(Oggetto.SASSO);
        partita.GestisciMano(mano);
        if (mano.GiocataComputer == Oggetto.CARTA) {
            VerificaPunteggi(pareggi, ++vittorieComputer, vittorieGiocatore);
        } else if (mano.GiocataComputer == Oggetto.FORBICI) {
            VerificaPunteggi(pareggi, vittorieComputer, ++vittorieGiocatore);
        } else if (mano.GiocataComputer == Oggetto.SASSO) {
            VerificaPunteggi(++pareggi, vittorieComputer, vittorieGiocatore);
        }        
    }
    
    private void VerificaPunteggi(int pareggi, int vittorieComputer, int vittorieGiocatore) {
        Assert.IsTrue(partita.Pareggi == pareggi, "pareggi");
        Assert.IsTrue(partita.ManiVinteDalComputer == vittorieComputer, "mani vinte dal computer");
        Assert.IsTrue(partita.ManiVinteDalGiocatore == vittorieGiocatore, "mani vinte dal giocatore");        
    }

}

}
